package ru.littlevictor.radio;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.graphics.RectF;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.LayoutInflater;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class CircleProgressBar extends RelativeLayout {
	
	private static final int PAINT_CLEAR = 0;
	private static final int PAINT_DRAW = 1;
	private static final int PAINT_FILL = 2;
	private int paintFlag = PAINT_CLEAR;
	private int progressbarSize;
	private int barBackgroundColor = getResources().getColor(android.R.color.darker_gray);
	private int barForegroundColor = getResources().getColor(android.R.color.holo_blue_light);
	private int progressbarCurrentValue = 0;
	private int progressbarPercentValue = 0;
	private int progressbarMaxValue = 360;
	
	private Paint paint;
	private Paint clearPaint;
	private Bitmap bitmap;
	private Canvas tmpCanvas;
	private DisplayMetrics metrics = getResources().getDisplayMetrics();
	private RelativeLayout progressbarRoot;
	private TextView progressbarText;

	public CircleProgressBar(Context context) {
		super(context);
		setWillNotDraw(false);
		init(context);
	};
	
	// -----------------------------------------------------------------------------
	public CircleProgressBar(Context context, AttributeSet attrs) {
		super(context, attrs);
		setWillNotDraw(false);
		init(context);
	};
	
	// -----------------------------------------------------------------------------
	public CircleProgressBar(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		setWillNotDraw(false);
		init(context);
	};
	
	// -----------------------------------------------------------------------------
	private void init(Context context) {
        LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        inflater.inflate(R.layout.circle_progress_bar, this, true);
        progressbarRoot = (RelativeLayout) findViewById(R.id.progressbar_root);
        progressbarText = (TextView) findViewById(R.id.progressbar_text);
        if (progressbarText != null) progressbarText.setText("0%");
        paint = new Paint();
        paint.setAntiAlias(true);
        clearPaint = new Paint(Paint.FILTER_BITMAP_FLAG);
        clearPaint.setColor(Color.TRANSPARENT);
        clearPaint.setAntiAlias(true);
        clearPaint.setXfermode(new PorterDuffXfermode(PorterDuff.Mode.CLEAR));
        setValue(25);
    };
	// -----------------------------------------------------------------------------
    @SuppressLint("DrawAllocation")
	@Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        int bigRadius = (int)(progressbarSize / 2);
        int smallRadius = (int)(progressbarSize / 2.5f);
        bitmap = Bitmap.createBitmap(canvas.getWidth(), canvas.getHeight(), Config.ARGB_8888);
        tmpCanvas = new Canvas(bitmap);
        switch(paintFlag) {
        case PAINT_CLEAR:
            tmpCanvas.drawColor(Color.TRANSPARENT);
            paint.setColor(barBackgroundColor);
            tmpCanvas.drawCircle(bigRadius, bigRadius, bigRadius, paint);
            tmpCanvas.drawCircle(bigRadius, bigRadius, smallRadius, clearPaint);
            break;
        case PAINT_DRAW:
        	float onePercent = progressbarMaxValue / 100;
        	progressbarPercentValue = (int) (progressbarCurrentValue / onePercent);
            paint.setColor(barBackgroundColor);
            tmpCanvas.drawCircle(bigRadius, bigRadius, bigRadius, paint);
            paint.setColor(barForegroundColor);
            tmpCanvas.drawArc(new RectF(0, 0, progressbarSize, progressbarSize), 270, (int)(progressbarPercentValue * 3.6f), true, paint);
            tmpCanvas.drawCircle(bigRadius, bigRadius, smallRadius, clearPaint);
			progressbarText.setText((int)(progressbarPercentValue) + "%");
            break;
        case PAINT_FILL:
            tmpCanvas.drawColor(Color.TRANSPARENT);
            paint.setColor(barForegroundColor);
            tmpCanvas.drawArc(new RectF(0, 0, progressbarSize, progressbarSize), 270, 360, true, paint);
            tmpCanvas.drawCircle(bigRadius, bigRadius, smallRadius, clearPaint);
            break;
        } 
        canvas.drawBitmap(bitmap, 0, 0, null);
    };
    
	// -----------------------------------------------------------------------------
	@Override
	protected void onSizeChanged(int width, int height, int xOld, int yOld){
		super.onSizeChanged(width, height, xOld, yOld);
		Log.d("Spinner", "width = " + width + " height = " + height);
		if(progressbarRoot != null) {
			LayoutParams params = (LayoutParams) progressbarRoot.getLayoutParams();
			if(width > height) {
				params.width = height;
				params.height = height;
				progressbarRoot.setLayoutParams(params);
			} else if(width < height) {
				params.width = width;
				params.height = width;
				progressbarRoot.setLayoutParams(params);
			}
			progressbarSize = width;
		if(progressbarText != null) progressbarText.setTextSize((int)(progressbarSize / (metrics.density * 3.5f + 0.5f)));
		} else {
			progressbarSize = 96;
		}
		Log.d("Spinner", "progressbarSize = " + progressbarSize);
	};
	
	// -----------------------------------------------------------------------------
	public void clear() {
		progressbarText.setText("0%");
		progressbarCurrentValue = 0;
		paintFlag = PAINT_CLEAR;
        invalidate();
	};
	
	// -----------------------------------------------------------------------------
	public void fill() {
		progressbarText.setText("100%");
		progressbarCurrentValue = progressbarMaxValue;
		paintFlag = PAINT_FILL;
        invalidate();
	};
	
	// -----------------------------------------------------------------------------
	public int getValue() {
		return progressbarCurrentValue;
	};
	
	// -----------------------------------------------------------------------------
	public int getMax() {
		return progressbarMaxValue;
	};
	
	// -----------------------------------------------------------------------------
	public void setTextColor(int color) {
		progressbarText.setTextColor(color);
	};
	
	// -----------------------------------------------------------------------------
	public void setForegroundColor(int color) {
		barForegroundColor = color;
		setValue(progressbarCurrentValue);
	};
	
	// -----------------------------------------------------------------------------
	public void setBackgroundColor(int color) {
		barBackgroundColor = color;
		setValue(progressbarCurrentValue);
	};
	
	// -----------------------------------------------------------------------------
	public void setMax(int value) {
			progressbarMaxValue = value;
	};
	
	// -----------------------------------------------------------------------------
	public boolean setValue(int value) {
		if(value >= progressbarMaxValue) {
			fill();
			return false;
		} else if (value <= 0) {
			clear();
			return false;
		} else {
			progressbarCurrentValue = value;
			paintFlag = PAINT_DRAW;
	        invalidate();
			return true;
		}
	};
};
