package ru.littlevictor.radio;

//-----------------------------------------------------------------------------
import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.Configuration;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.ActionBarActivity;
import android.support.v7.widget.Toolbar;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.DisplayMetrics;
import android.util.SparseBooleanArray;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EdgeEffect;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.TreeMap;

//import android.util.Log;

//-----------------------------------------------------------------------------
@SuppressLint({ "DefaultLocale" })
public class DialogFile extends ActionBarActivity {
	// -----------------------------------------------------------------------------
    private final int DIALOG_NONE = 0;
    private final int DIALOG_NEW_FOLDER = 1;
    private int dialogIsShow = DIALOG_NONE;
	private final String ITEM_KEY = "key";
	private final String ITEM_IMAGE = "image";
	private final String ROOT = "/";
	public static final String SAVED_NAME = "SAVED_NAME";
	public static final String START_PATH = "START_PATH";
	public static final String FORMAT_FILTER = "FORMAT_FILTER";
	public static final String RESULT_PATH = "RESULT_PATH";
	public static final String SELECTION_MODE = "SELECTION_MODE";
	public static final String CAN_SELECT_DIR = "CAN_SELECT_DIR";
	public static final int MODE_CREATE = 0;
	public static final int MODE_OPEN = 1;
	public static final int MODE_SELECT = 2;
	public static final int MODE_SELECT_DIR = 3;
    private int editPosition = -1;
    private int[] theme;
    private int themeBaseColor;
    private int selectionMode = MODE_OPEN;
    private int orientation;
    private boolean canSelectDir = false;
    private boolean pause = false;
	private List<String> path;
	private EditText mFileName;
	private TextView location;
	private ArrayList<HashMap<String, Object>> mList = null;
	private Button openSaveButton;
	private Button cancelButton;
	private LinearLayout fileNameLayout;
	private InputMethodManager inputManager;
    private String newFolderName = "";
	private String fileName = "";
	private String savedName = "";
	private String parentPath = "";
	private String currentPath = ROOT;
	private String[] formatFilter = null;
	private File selectedFile = null;
	private HashMap<String, Integer> lastPositions = new HashMap<String, Integer>();
	private AdapterFileList fileList = null;
	private ListView list;
    private Toolbar actionBar;
    private ActionBarFragment actionBarFragment;
    private FragmentTransaction transaction;
    private AlertDialog mDialog;
    private ViewGroup.LayoutParams layoutParams;

	// -----------------------------------------------------------------------------
	@Override
	public void onCreate(Bundle savedInstanceState) {
//        Log.d("myLog", "DialogFile onCreate");
        theme = ThemeChooser.getAppTheme(this);
        themeBaseColor = getResources().getColor(theme[0]);
        setTheme(theme[2]);
        super.onCreate(savedInstanceState);
        selectionMode = getIntent().getIntExtra(SELECTION_MODE,	MODE_OPEN);
        formatFilter = getIntent().getStringArrayExtra(FORMAT_FILTER);
        canSelectDir = getIntent().getBooleanExtra(CAN_SELECT_DIR, false);
        setContentView(R.layout.dialog_file);
        createActionBar(getResources().getConfiguration().orientation);
        list = (ListView) findViewById(R.id.fdList);
        brandGlowEffect(this, getResources().getColor(theme[1]));
        LinearLayout l = (LinearLayout) findViewById(R.id.fdBottomPanel);
        l.setBackgroundColor(themeBaseColor);
		setResult(RESULT_CANCELED, getIntent());
		currentPath = Environment.getExternalStorageDirectory().toString();
		
		fileNameLayout = (LinearLayout) findViewById(R.id.fdFilePanel);
		list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                inputManager.hideSoftInputFromWindow(view.getWindowToken(), 0);
                File file = new File(path.get(position));
                if (file.isDirectory()) {
                    if (file.canRead()) {

                        if(selectionMode == MODE_CREATE) {
                            if(mFileName.getText().toString().length() > 0) {
                                openSaveButton.setEnabled(true);
                            } else {
                                openSaveButton.setEnabled(false);
                            }
                        } else {
                            openSaveButton.setEnabled(false);
                        }

                        lastPositions.put(currentPath, position);
                        getDir(path.get(position));
                        if (canSelectDir) {
                            selectedFile = file;
                            openSaveButton.setEnabled(true);
                        };
                    } else {
                        openSaveButton.setEnabled(false);
                        list.clearChoices();
                        list.requestLayout();
                        showToast(getResources().getString(R.string.access_denied) + "\n" + file.toString(), Toast.LENGTH_LONG);
                    };
                } else {
                    fileName = file.getName();
                    mFileName.setText(fileName);
                    selectedFile = file;
                    openSaveButton.setEnabled(true);
                }
            }
        });
        mFileName = (EditText) findViewById(R.id.fdFile);
		location = (TextView) findViewById(R.id.fdPath);
        openSaveButton = (Button) findViewById(R.id.fdButton);
		cancelButton = (Button) findViewById(R.id.fdButtonCancel);
		inputManager = (InputMethodManager) getSystemService(INPUT_METHOD_SERVICE);
		mFileName.addTextChangedListener(new TextWatcher() {
	        public void afterTextChanged(Editable s) {
				if(!fileName.equalsIgnoreCase(s.toString())) {
		        	list.clearChoices();
					list.requestLayout();
				}
	        	if(s.length() > 0) {
	        		openSaveButton.setEnabled(true);
	        	} else {
	        		openSaveButton.setEnabled(false);
	        	}
	        };
	        public void beforeTextChanged(CharSequence s, int start, int count, int after){};
	        public void onTextChanged(CharSequence s, int start, int before, int count){};
	    }); 
		switch(selectionMode) {
		case MODE_OPEN:
			setTitle(R.string.action_open);
			fileNameLayout.setVisibility(View.GONE);
			openSaveButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					if (selectedFile != null) returnResult();
				};
			});
			break;
		case MODE_CREATE:
			savedName = getIntent().getStringExtra(SAVED_NAME);
			if(savedName != null && savedName.length() > 0) {
				mFileName.setText(savedName);
        		openSaveButton.setEnabled(true);
			}
			setTitle(R.string.action_save);
			fileNameLayout.setVisibility(View.VISIBLE);
			openSaveButton.setText(R.string.btn_save);
			openSaveButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					if (mFileName.getText().length() > 0) returnResult();
				};
			});
			break;
		case MODE_SELECT:
			setTitle(R.string.action_select);
			fileNameLayout.setVisibility(View.GONE);
			openSaveButton.setText(R.string.btn_select);
			openSaveButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					if (selectedFile != null) returnResult();
				};
			});
			break;
		case MODE_SELECT_DIR:
			setTitle(R.string.action_select_dir);
			fileNameLayout.setVisibility(View.GONE);
			openSaveButton.setText(R.string.btn_select);
			openSaveButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					if (selectedFile != null) {
						setResult(RESULT_OK, new Intent().setData(Uri.parse("file://" + currentPath)));
						finish();
					}
				};
			});
			break;
		}
		cancelButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				finish();
			};
		});
		String startPath = getIntent().getStringExtra(START_PATH);
		startPath = startPath != null ? startPath : currentPath;
		if (canSelectDir) {
			File file = new File(startPath);
			selectedFile = file;
			openSaveButton.setEnabled(true);
		};
		getDir(startPath);
	};

	// -----------------------------------------------------------------------------
	private void returnResult() {
		setResult(RESULT_OK, new Intent().setData(Uri.parse("file://" + currentPath + "/" + mFileName.getText())));
		finish();
	};
	
	// -----------------------------------------------------------------------------
	private void getDir(String dirPath) {
		boolean useAutoSelection = dirPath.length() < currentPath.length();
		Integer position = lastPositions.get(parentPath);
		getDirImpl(dirPath);
		if (position != null && useAutoSelection) {
            list.setSelection(position);
		}
	};

	// -----------------------------------------------------------------------------
	private void getDirImpl(final String dirPath) {
		currentPath = dirPath;
		final List<String> item = new ArrayList<String>();
		path = new ArrayList<String>();
		mList = new ArrayList<HashMap<String, Object>>();
		File f = new File(currentPath);
		File[] files = f.listFiles();
		if (files == null) {
			currentPath = ROOT;
			f = new File(currentPath);
			files = f.listFiles();
		}
		Arrays.sort(files);
		location.setText(currentPath);
		if (!currentPath.equals(ROOT)) {

			item.add(ROOT);
			addItem(ROOT, R.drawable.folder);
			path.add(ROOT);

			item.add("../");
			addItem("../", R.drawable.folder);
			path.add(f.getParent());
			parentPath = f.getParent();

		}
		TreeMap<String, String> dirsMap = new TreeMap<String, String>();
		TreeMap<String, String> dirsPathMap = new TreeMap<String, String>();
		TreeMap<String, String> filesMap = new TreeMap<String, String>();
		TreeMap<String, String> filesPathMap = new TreeMap<String, String>();
		for (File file : files) {
			if (file.isDirectory()) {
				String dirName = file.getName();
				dirsMap.put(dirName, dirName);
				dirsPathMap.put(dirName, file.getPath());
			} else {
				final String fileName = file.getName();
				final String fileNameLwr = fileName.toLowerCase();
				if (formatFilter != null) {
					boolean contains = false;
					for (int i = 0; i < formatFilter.length; i++) {
						final String formatLwr = formatFilter[i].toLowerCase();
						if (fileNameLwr.endsWith(formatLwr)) {
							contains = true;
							break;
						}
					}
					if (contains) {
						filesMap.put(fileName, fileName);
						filesPathMap.put(fileName, file.getPath());
					}
				} else {
					filesMap.put(fileName, fileName);
					filesPathMap.put(fileName, file.getPath());
				}
			}
		}
		item.addAll(dirsMap.tailMap("").values());
		item.addAll(filesMap.tailMap("").values());
		path.addAll(dirsPathMap.tailMap("").values());
		path.addAll(filesPathMap.tailMap("").values());
		fileList = new AdapterFileList(this, mList, R.layout.dialog_file_cell,
                new String[] { ITEM_KEY, ITEM_IMAGE },
                new int[] { R.id.fdrowtext, R.id.fdrowimage });

		for (String dir : dirsMap.tailMap("").values()) {
			addItem(dir, R.drawable.folder);
		}

		for (String file : filesMap.tailMap("").values()) {
			addItem(file, R.drawable.file);
		}
		fileList.notifyDataSetChanged();
        list.setAdapter(fileList);
	};

	// -----------------------------------------------------------------------------
	private void addItem(String fileName, int imageId) {
		HashMap<String, Object> item = new HashMap<String, Object>();
		item.put(ITEM_KEY, fileName);
		item.put(ITEM_IMAGE, imageId);
		mList.add(item);
	};

	// -----------------------------------------------------------------------------
	private void showNewFolderDialog() {
        final LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_prompt, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
		TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.dialog_new_folder_title));
        layoutParams = dialogTitle.getLayoutParams();
        if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
            layoutParams.width = (int)(getScreenMaxPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        } else {
            layoutParams.width = (int)(getScreenMinPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        }
        final EditText dialogName = (EditText) dialog.findViewById(R.id.dialog_name);
		dialogName.setText(newFolderName);
        final Button btnOk = (Button) dialog.findViewById(R.id.btn_ok);
        btnOk.setEnabled(false);
        btnOk.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                inputManager.hideSoftInputFromWindow((dialog).getWindowToken(), 0);
                File folder = new File(currentPath + "/" + dialogName.getText());
                boolean success = true;
                if (!folder.exists()) {
                    success = folder.mkdir();
                }
                if (success) {
                    getDir(currentPath);
                } else {
                    showToast(getResources()
                                    .getString(R.string.access_denied) + "\n" + currentPath,
                            Toast.LENGTH_LONG);
                }
            };
        });
        Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                inputManager.hideSoftInputFromWindow((dialog).getWindowToken(), 0);
            };
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
            };
        });
        mDialog.show();
		dialogName.addTextChangedListener(new TextWatcher() {
	        public void afterTextChanged(Editable s) {
	        	if(s.length() > 0) {
                    newFolderName = s.toString();
                    btnOk.setEnabled(true);
	        	} else {
                    btnOk.setEnabled(false);
	        	}
	        };
	        public void beforeTextChanged(CharSequence s, int start, int count, int after){};
	        public void onTextChanged(CharSequence s, int start, int before, int count){};
	    });
        dialogIsShow = DIALOG_NEW_FOLDER;
	};

    // -----------------------------------------------------------------------------
    private void brandGlowEffect(Context context, int brandColor) {
        if (Build.VERSION.SDK_INT >= 21)
        {
            EdgeEffect edgeEffectTop = new EdgeEffect(this);
            edgeEffectTop.setColor(brandColor);

            EdgeEffect edgeEffectBottom = new EdgeEffect(this);
            edgeEffectBottom.setColor(brandColor);

            try {
                Field f1 = AbsListView.class.getDeclaredField("mEdgeGlowTop");
                f1.setAccessible(true);
                f1.set(list, edgeEffectTop);

                Field f2 = AbsListView.class.getDeclaredField("mEdgeGlowBottom");
                f2.setAccessible(true);
                f2.set(list, edgeEffectBottom);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        else {
            int glowDrawableId = context.getResources().getIdentifier("overscroll_glow", "drawable", "android");
            Drawable androidGlow = context.getResources().getDrawable(glowDrawableId);
            androidGlow.setColorFilter(brandColor, PorterDuff.Mode.SRC_IN);
            int edgeDrawableId = context.getResources().getIdentifier("overscroll_edge", "drawable", "android");
            Drawable androidEdge = context.getResources().getDrawable(edgeDrawableId);
            androidEdge.setColorFilter(brandColor, PorterDuff.Mode.SRC_IN);
        }
    };

	// -----------------------------------------------------------------------------
	private void showToast(String message, int length) {
		Toast toast = Toast.makeText(this, message, length);
		toast.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL, 0, 0);
		toast.show();
	};

	// -----------------------------------------------------------------------------
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
//        Log.d("myLog", "DialogFile onCreateOptionsMenu");
        actionBar = actionBarFragment.getActionBar();
        actionBar.setNavigationIcon(R.drawable.abc_ic_ab_back_mtrl_am_alpha);
		if (selectionMode == MODE_CREATE) {
            getMenuInflater().inflate(R.menu.dialog_file, menu);
		}
        if(editPosition > 0 && editPosition < list.getCount()) {
            list.setItemChecked(editPosition, true);
        }
		return true;
	};

    // -----------------------------------------------------------------------------
    private void createActionBar(int orientation) {
//        Log.d("myLog", "DialogFile createActionBar");
        int subtitleId;
        switch(selectionMode) {
            case MODE_OPEN:
                subtitleId = R.string.action_open;
                break;
            case MODE_CREATE:
                subtitleId = R.string.action_save;
                break;
            case MODE_SELECT:
                subtitleId = R.string.action_select;
                break;
            case MODE_SELECT_DIR:
                subtitleId = R.string.action_select_dir;
                break;
            default:
                subtitleId = R.string.action_open;
                break;
        }
        actionBarFragment = new ActionBarFragment();
        actionBarFragment.setParentActivity(this);
        Bundle args = new Bundle();
        if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
            args.putInt(actionBarFragment.LAYOUT_ORIENTATION, actionBarFragment.LAYOUT_LANDSCAPE);
        } else {
            args.putInt(actionBarFragment.LAYOUT_ORIENTATION, actionBarFragment.LAYOUT_PORTRAIT);
        }
        args.putString(actionBarFragment.TITLE, getResources().getString(R.string.app_name));
        args.putString(actionBarFragment.SUBTITLE, getResources().getString(subtitleId));
        actionBarFragment.setArguments(args);
        transaction = getSupportFragmentManager().beginTransaction();
        transaction.replace(R.id.action_bar_container, actionBarFragment);
        transaction.commit();
    };

	//-----------------------------------------------------------------------------
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch(item.getItemId()) {
        case R.id.action_add:
        	showNewFolderDialog();
			return true;
        case android.R.id.home:
			finish();
			return true;
		}
		return super.onOptionsItemSelected(item);
	};
	
	// -----------------------------------------------------------------------------
	public boolean onKeyDown(int keyCode, KeyEvent event) {
        switch (keyCode) { 
        case KeyEvent.KEYCODE_BACK:
			finish();
			return true;
        } 
		return super.onKeyDown(keyCode, event);
	};

    // -----------------------------------------------------------------------------
    private int getScreenMaxPixels() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height >= width) {
            return height;
        } else {
            return width;
        }
    };

    // -----------------------------------------------------------------------------
    private int getScreenMinPixels() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height <= width) {
            return height;
        } else {
            return width;
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onSaveInstanceState(Bundle outState) {
//        Log.d("myLog", "RadioMain onSaveInstanceState");
        super.onSaveInstanceState(outState);
        SparseBooleanArray checkedSparse = list.getCheckedItemPositions();
        int len = list.getCount();
        boolean[] checked = new boolean[len];
        for (int i = 0; i < len; i++) {
            if (checkedSparse.get(i)) {
                checked[i] = true;
            } else {
                checked[i] = false;
            }
        }
        outState.putBooleanArray("checked", checked);
        outState.putInt("dialogIsShow", dialogIsShow);
        outState.putString("currentPath", currentPath);
        outState.putString("mFileName", mFileName.getText().toString());
        outState.putString("newFolderName", newFolderName);
        outState.putBoolean("openSaveButton", openSaveButton.isEnabled());
        if(selectedFile != null) {
            if(selectedFile.isFile()) {
                outState.putString("selectedFile", selectedFile.getAbsolutePath());
            }
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
//        Log.d("myLog", "RadioMain onRestoreInstanceState");
        super.onRestoreInstanceState(savedInstanceState);
        if(savedInstanceState != null) {
            boolean[] checked = savedInstanceState.getBooleanArray("checked");
            if(checked != null) {
                for (int i = 0; i < checked.length; i++) {
                    if (checked[i] == true) {
                        editPosition = i;
                    }
                }
            }
            getDir(savedInstanceState.getString("currentPath", currentPath));
            mFileName.setText(savedInstanceState.getString("mFileName", ""));
            newFolderName = savedInstanceState.getString("newFolderName", "");
            openSaveButton.setEnabled(savedInstanceState.getBoolean("openSaveButton"));
            String selectedFilePath = savedInstanceState.getString("selectedFile");
            if(selectedFilePath != null) {
                selectedFile = new File(selectedFilePath);
            }
            switch(savedInstanceState.getInt("dialogIsShow", DIALOG_NONE)) {
                case DIALOG_NEW_FOLDER:
                    showNewFolderDialog();
                    break;
            }
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onDestroy() {
        if(mDialog != null) {
            mDialog.dismiss();
        }
        dialogIsShow = DIALOG_NONE;
        super.onDestroy();
//        Log.d("myLog", "RadioMain onDestroy");
    };
};