package ru.littlevictor.radio;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

public class DialogNetwork extends Activity {
    private final int DIALOG_NONE = 0;
    private final int DIALOG_NETWORK = 1;
    private int dialogIsShow = DIALOG_NONE;
    private AlertDialog mDialog = null;
    private int[] theme;

	@Override
    public void onCreate(Bundle savedInstanceState) {
        theme = ThemeChooser.getAppTheme(this);
        setTheme(R.style.DialogLightTheme);
        super.onCreate(savedInstanceState);
        if(savedInstanceState != null) {
            dialogIsShow = savedInstanceState.getInt("dialogIsShow", DIALOG_NONE);
            switch(dialogIsShow) {
                case DIALOG_NETWORK:
                    showNetworkDialog();
                    break;
            }

        } else {
            showNetworkDialog();
        }
    };

    // -----------------------------------------------------------------------------
    private void showNetworkDialog() {
        LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_alert, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
        TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.dialog_network_title));
        TextView dialogMessage = (TextView) dialog.findViewById(R.id.dialog_message);
        dialogMessage.setText(getText(R.string.dialog_network_message));
        Button btnOk = (Button) dialog.findViewById(R.id.btn_ok);
        btnOk.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                finish();
                startActivity(new Intent(android.provider.Settings.ACTION_SETTINGS));
            };
        });
        Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                finish();
            };
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
                finish();
            };
        });
        mDialog.show();
        dialogIsShow = DIALOG_NETWORK;
    };

	// -----------------------------------------------------------------------------
	public boolean onKeyDown(int keyCode, KeyEvent event) {
        switch (keyCode) { 
        case KeyEvent.KEYCODE_BACK:
            if(mDialog != null) mDialog.dismiss();
			finish();
        }
		return super.onKeyDown(keyCode, event);
	};

    // -----------------------------------------------------------------------------
    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt("dialogIsShow", dialogIsShow);
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onDestroy() {
        if(mDialog != null) mDialog.dismiss();
        super.onDestroy();
    };
};
