package ru.littlevictor.radio;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.Configuration;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.PowerManager;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Arrays;

public class RadioListCreator extends Activity {
    private final int DIALOG_NONE = 0;
    private final int DIALOG_PROGRESS = 1;
    private final int DIALOG_LIST_CREATE_ERROR = 2;
    private int dialogIsShow = DIALOG_NONE;
    private int[] theme;
    private int progressBgColor;
    private int themeAccentColor;
    public static boolean running = false;
    private AlertDialog mDialog;
    private MyTask listCreateTask;
    private LayoutParams layoutParams;
    private CircleProgressBar primaryProgressBar;
    private CircleProgressBar secondaryProgressBar;
    private TextView primaryProgressBarText;
    private TextView secondaryProgressBarText;
    private RelativeLayout primaryContent;
    private String errorMessage = "";

    // -----------------------------------------------------------------------------
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        theme = ThemeChooser.getAppTheme(this);
        themeAccentColor = getResources().getColor(theme[0]);
        progressBgColor = getResources().getColor(R.color.color_background_primary_light_selected);
        super.onCreate(savedInstanceState);
        if(savedInstanceState != null) {
            dialogIsShow = savedInstanceState.getInt("dialogIsShow", DIALOG_NONE);
            switch(dialogIsShow) {
                case DIALOG_PROGRESS:
                    if(running) {
                        showProgressDialog();
                        if(savedInstanceState.getInt("primaryProgressBarValue", View.GONE) == View.VISIBLE) {
                            primaryContent.setVisibility(View.VISIBLE);
                        }
                        primaryProgressBar.setValue(savedInstanceState.getInt("primaryProgressBarValue", 0));
                        primaryProgressBarText.setText(savedInstanceState.getString("primaryProgressBarText", ""));
                        secondaryProgressBar.setValue(savedInstanceState.getInt("secondaryProgressBarValue", 0));
                        secondaryProgressBarText.setText(savedInstanceState.getString("secondaryProgressBarText", ""));
                    }
                    break;
                case DIALOG_LIST_CREATE_ERROR:
                    showListCreateErrorDialog(savedInstanceState.getString("errorMessage", errorMessage));
                    break;
            }
        }
        String path = "";
        Uri inputData = getIntent().getData();
        if (inputData != null) {
            try {
                path = inputData.getPath();
            } catch (Exception e) {
                showToast(getResources().getString(R.string.invalid_file_path), Toast.LENGTH_LONG);
                finish();
            }
            if(savedInstanceState == null) {
                running = true;
                showProgressDialog();
            }
            listCreateTask = new MyTask(this);
            listCreateTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, path);
        } else {
            showToast(getResources().getString(R.string.invalid_file_path), Toast.LENGTH_LONG);
            finish();
        }
    };

    // -----------------------------------------------------------------------------
    private class MyTask extends AsyncTask<String, Integer, String> {
        private Context context;
        private PowerManager.WakeLock mWakeLock;
        private String secondaryPath = "";
        private String secondaryName = "";
        private String childFileName = "";
        private String rootFileContent = "";
        private String rootFileName = "";
        private String rootFilePath = "";
        private int primaryCount = 0;
        private int primaryMaxCount = 3600;
        private int primaryProgress = 0;
        private int secondaryMaxCount = 0;
        private String result = null;
        private final String[] mediaFileExt = getResources().getStringArray(R.array.media_file_extensions_array);
        private final String[] playlistFileExt = getResources().getStringArray(R.array.playlist_extensions_array);
        private boolean rootDirIsNotEmpty = false;

        // -----------------------------------------------------------------------------
        public MyTask(Context context) {
            this.context = context;
        };

        public boolean isSymlink(File file) throws IOException {
            File canon;
            if (file.getParent() == null) {
                canon = file;
            } else {
                File canonDir = file.getParentFile().getCanonicalFile();
                canon = new File(canonDir, file.getName());
            }
            return !canon.getCanonicalFile().equals(canon.getAbsoluteFile());
        };

        // -----------------------------------------------------------------------------
        private boolean isMediaFile(String name) {
            String nameLwr = name.toLowerCase();
            boolean contains = false;
            for (int i = 0; i < mediaFileExt.length; i++) {
                if (nameLwr.endsWith(mediaFileExt[i])) {
                    contains = true;
                    break;
                }
            }
            return contains;
        };

        // -----------------------------------------------------------------------------
        private boolean isPlaylistFile(String name) {
            String nameLwr = name.toLowerCase();
            boolean contains = false;
            for (int i = 0; i < playlistFileExt.length; i++) {
                if (nameLwr.endsWith(playlistFileExt[i])) {
                    contains = true;
                    break;
                }
            }
            return contains;
        };

        // -----------------------------------------------------------------------------
        private void countFiles(String dirPath) {
            File[] files;
            try {
                File dir = new File(dirPath);
                files = dir.listFiles();
                if(files == null) return;
            } catch (Exception e) {
                return;
            }
            primaryCount = files.length;
            primaryMaxCount = primaryCount * 360;
            for (File file : files) {
                if (file.isDirectory()) {
                    Integer[] progress = {0, 0};
                    publishProgress(progress);
                    break;
                }
            }
        };

        // -----------------------------------------------------------------------------
        private boolean createList(String dirPath) {
            if(dirPath.equalsIgnoreCase("/proc") || dirPath.equalsIgnoreCase("/dev") || dirPath.equalsIgnoreCase("/sys") || dirPath.equalsIgnoreCase("/d")) {
                return false;
            }
            boolean currentDirIsNotEmpty = false;
            boolean isRootDir;
            String str = "";
            String parentPrefix = "";
            File dir;
            File parentDir;
            File[] files;
            try {
                dir = new File(dirPath);
                if(isSymlink(dir))return false;
                files = dir.listFiles();
                if(files == null) return false;
            } catch (Exception e) {
                return false;
            }
            if(dirPath.equalsIgnoreCase("/")) {
                parentDir = new File(dirPath);
            } else {
                parentDir = new File(dir.getParent());
            }
            Arrays.sort(files);
            int length = files.length;

            childFileName = dir.getName();
            secondaryMaxCount = length * 360;
            String childFilePath = dir.getPath();

            if(childFilePath.equalsIgnoreCase(rootFilePath)) {
                isRootDir = true;
                parentPrefix = "";
            } else {
                isRootDir = false;
                parentPrefix = parentDir.getName() + "_";
            }

            for (int count = 0; count < length; count++) {
                File file = files[count];
                if (file.isDirectory()) {
                    createList(file.getPath());
                    secondaryName = file.getName();
                    secondaryPath = file.getAbsolutePath();
                    childFileName = dir.getName();
                    secondaryMaxCount = length * 100;

                    if(isRootDir) {
                        primaryProgress = primaryProgress + (int)(primaryMaxCount / primaryCount);
                        Integer[] progress = {primaryProgress, (int)((secondaryMaxCount / length) * (count + 1))};
                        publishProgress(progress);
                    } else {
                        Integer[] progress = {-1, (int)((secondaryMaxCount / length) * (count + 1))};
                        publishProgress(progress);
                    }
                } else {
                    secondaryName = file.getName();
                    secondaryPath = file.getAbsolutePath();

                    if(isMediaFile(secondaryName)) {
                        rootDirIsNotEmpty = true;
                        currentDirIsNotEmpty = true;
                        if(isRootDir) {
                            if(isPlaylistFile(secondaryName)) {
                                parentPrefix = dir.getName() + "_";
                                rootFileContent += "#EXTINF:0," + secondaryName.substring(0, secondaryName.lastIndexOf(".")) + "\n";
                                rootFileContent += "file://" + file.getAbsolutePath() + "\n";
                            } else {
                                rootFileContent += "#EXTINF:0," + secondaryName.substring(0, secondaryName.lastIndexOf(".")) + "\n";
                                rootFileContent += "file://" + file.getAbsolutePath() + "\n";
                            }
                        } else {
                            if(isPlaylistFile(secondaryName)) {
                                rootFileContent += "#EXTINF:0," + secondaryName.substring(0, secondaryName.lastIndexOf(".")) + "\n";
                                rootFileContent += "file://" + file.getAbsolutePath() + "\n";
                            } else {
                                int dotPosition = secondaryName.lastIndexOf(".");
                                if(dotPosition == -1) {
                                    str += "#EXTINF:0," + secondaryName + "\n";
                                } else {
                                    str += "#EXTINF:0," + secondaryName.substring(0, dotPosition) + "\n";
                                }
                                str += "file://" + file.getAbsolutePath() + "\n";
                            }
                        }
                    }
                    if(isRootDir) {
                        primaryProgress = primaryProgress + (int)(primaryMaxCount / primaryCount);
                        Integer[] progress = {primaryProgress, (int)((secondaryMaxCount / length) * (count + 1))};
                        publishProgress(progress);
                    } else {
                        Integer[] progress = {-1, (int)((secondaryMaxCount / length) * (count + 1))};
                        publishProgress(progress);
                    }
                }
            }
            if(currentDirIsNotEmpty) {
                if(!isRootDir) {
                    if(!str.equalsIgnoreCase("")) {
                        str = "#EXTM3U\n" + str;
                        try {
                            BufferedWriter bw = new BufferedWriter(new FileWriter(Storage.getPath(context, Storage.TMP_DIR) + parentPrefix + childFileName + ".m3u"));
                            bw.write(str);
                            bw.close();
                            rootFileContent += "#EXTINF:0," + childFileName + "\n";
                            rootFileContent += "file://" + Storage.getPath(context, Storage.TMP_DIR) + parentPrefix + childFileName + ".m3u" + "\n";
                        } catch (IOException e) {
                            result = e.toString();
                            return false;
                        }
                    }
                }
            }
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            return rootDirIsNotEmpty;
        };

        // -----------------------------------------------------------------------------
        @Override
        protected String doInBackground(String... createDirPath) {
            File rootDir = new File(createDirPath[0]);
            rootFilePath = rootDir.getPath();
            if(rootFilePath.equalsIgnoreCase("/")) {
                rootFileName = "ROOT";
            } else {
                rootFileName = rootDir.getName();
            }
            rootFileContent = "#EXTM3U\n";
            countFiles(rootDir.getPath());
            if(createList(rootDir.getPath())) {
                try {
                    BufferedWriter bw = new BufferedWriter(new FileWriter(Storage.getPath(context, Storage.TMP_DIR) + rootFileName + ".m3u"));
                    bw.write(rootFileContent);
                    bw.close();
                } catch (IOException e) {
                    if (result == null) result = e.toString();
                }
            } else {
                if (result == null) result = getResources().getString(R.string.media_files_not_found);
            }
            return result;
        };

        // -----------------------------------------------------------------------------
        @Override
        protected void onProgressUpdate(Integer... progress) {
            super.onProgressUpdate(progress);
            if(dialogIsShow == DIALOG_PROGRESS) {
                if(progress[0] != -1) {
                    primaryContent.setVisibility(View.VISIBLE);
                    primaryProgressBar.setMax(primaryMaxCount);
                    primaryProgressBarText.setText(rootFileName);
                    primaryProgressBar.setValue(progress[0]);
                }
                if(progress.length > 1 && progress[1] != -1) {
                    secondaryProgressBar.setMax(secondaryMaxCount);
                    secondaryProgressBarText.setText(secondaryPath);
                    secondaryProgressBar.setValue(progress[1]);
                }
            }
        };

        // -----------------------------------------------------------------------------
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            PowerManager pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
            mWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, getClass().getName());
            mWakeLock.acquire();
        };

        // -----------------------------------------------------------------------------
        @Override
        protected void onPostExecute(String result) {
            running = false;
            mWakeLock.release();
            if(dialogIsShow == DIALOG_PROGRESS && mDialog != null) {
                dialogIsShow = DIALOG_NONE;
                mDialog.dismiss();
                mDialog = null;
            }
            if (result != null) {
                errorMessage = result;
                showListCreateErrorDialog(errorMessage);
            } else {
                setResult(RESULT_OK, getIntent().setData(Uri.parse("file://" + Storage.getPath(context, Storage.TMP_DIR) + rootFileName + ".m3u")));
                finish();
            }
        };

        // -----------------------------------------------------------------------------
        @Override
        protected void onCancelled() {
            running = false;
            super.onCancelled();
            if(dialogIsShow == DIALOG_PROGRESS && mDialog != null) {
                dialogIsShow = DIALOG_NONE;
                mDialog.dismiss();
                mDialog = null;
            }
            if (result != null) {
                errorMessage = result;
                showListCreateErrorDialog(errorMessage);
            } else {
                setResult(RESULT_CANCELED, getIntent());
                finish();
            }
        };
    };

    // -----------------------------------------------------------------------------
    private void showProgressDialog() {
        final LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.circle_progress_dialog, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
        final TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        layoutParams = dialogTitle.getLayoutParams();
        if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
            layoutParams.width = (int)(getScreenMaxPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        } else {
            layoutParams.width = (int)(getScreenMinPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        }
        primaryContent = (RelativeLayout) dialog.findViewById(R.id.primaryContent);
        primaryProgressBar = (CircleProgressBar) dialog.findViewById(R.id.primaryProgressBar);
        secondaryProgressBar = (CircleProgressBar) dialog.findViewById(R.id.secondaryProgressBar);
        primaryProgressBar.setForegroundColor(themeAccentColor);
        secondaryProgressBar.setForegroundColor(themeAccentColor);
        primaryProgressBar.setBackgroundColor(progressBgColor);
        secondaryProgressBar.setBackgroundColor(progressBgColor);
        primaryProgressBarText = (TextView) dialog.findViewById(R.id.primaryDialogText);
        secondaryProgressBarText = (TextView) dialog.findViewById(R.id.secondaryDialogText);
        final Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                mDialog.dismiss();
                mDialog = null;
                listCreateTask.cancel(true);
                setResult(RESULT_CANCELED, getIntent());
                finish();
            }
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(false);
        mDialog = builder.create();
        mDialog.show();
        dialogIsShow = DIALOG_PROGRESS;
    };

    // -----------------------------------------------------------------------------
    public void showListCreateErrorDialog(String msg) {
        LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_message, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
        final TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.dialog_create_playlist_error_title));
        layoutParams = dialogTitle.getLayoutParams();
        if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
            layoutParams.width = (int)(getScreenMaxPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        } else {
            layoutParams.width = (int)(getScreenMinPixels()/100*90);
            dialogTitle.setLayoutParams(layoutParams);
        }
        TextView dialogMessage = (TextView) dialog.findViewById(R.id.dialog_message);
        dialogMessage.setText(msg);
        Button btnOk = (Button) dialog.findViewById(R.id.btn_ok);
        btnOk.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                setResult(RESULT_CANCELED, getIntent());
                finish();
            };
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
                setResult(RESULT_CANCELED, getIntent());
                finish();
            };
        });
        mDialog.show();
        dialogIsShow = DIALOG_LIST_CREATE_ERROR;
    };

    // -----------------------------------------------------------------------------
    private void showToast(String message, int length) {
        Toast toast = Toast.makeText(this, message, length);
        toast.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL, 0, 0);
        toast.show();
    };

    // -----------------------------------------------------------------------------
    private int getScreenMaxPixels() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height >= width) {
            return height;
        } else {
            return width;
        }
    };

    // -----------------------------------------------------------------------------
    private int getScreenMinPixels() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height <= width) {
            return height;
        } else {
            return width;
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt("dialogIsShow", dialogIsShow);
        switch(dialogIsShow) {
            case DIALOG_PROGRESS:
                outState.putString("primaryProgressBarText", primaryProgressBarText.getText().toString());
                outState.putString("secondaryProgressBarText", secondaryProgressBarText.getText().toString());
                outState.putInt("primaryProgressBarValue", primaryProgressBar.getValue());
                outState.putInt("secondaryProgressBarValue", secondaryProgressBar.getValue());
                outState.putInt("primaryContentVisibility", primaryContent.getVisibility());
                break;
            case DIALOG_LIST_CREATE_ERROR:
                outState.putString("errorMessage", errorMessage);
                break;
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onDestroy() {
        dialogIsShow = DIALOG_NONE;
        if(mDialog != null) {
            mDialog.dismiss();
        }
        super.onDestroy();
    };

}
