package ru.littlevictor.radio;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PorterDuff;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.media.AudioManager;
import android.media.SoundPool;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.ActionBarActivity;
import android.support.v7.widget.Toolbar;
import android.telephony.TelephonyManager;
import android.util.DisplayMetrics;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnLongClickListener;
import android.view.ViewGroup.LayoutParams;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EdgeEffect;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.lang.reflect.Field;
import java.util.ArrayList;

public class RadioMain extends ActionBarActivity {
	//-----------------------------------------------------------------------------
    public static boolean running = false;
    private boolean headsetBeepEnable = false;
	private final int REQUEST_SAVE = 0;
	private final int REQUEST_OPEN = 1;
	private final int REQUEST_EDIT = 2;
	private final int REQUEST_CREATE = 3;
    private final int REQUEST_GET_LIST = 4;
    private final int DIALOG_NONE = 0;
    private final int DIALOG_SAVE_AS_MAIN = 1;
    private final int DIALOG_CHOOSE_ACTION = 2;
    private final int DIALOG_CODE_PAGE = 4;
    private int dialogIsShow = DIALOG_NONE;
    private int editPosition = 0;
    private int serviceState = RadioService.TERM;
    private int clickCount = 0;
    private int[] theme;
    private int themeBaseColor;
    private int themeAccentColor;
	private int currentPosition;
	private int orientation;
	private int trackHeight = 0;	
	private int first = 0;
	private int last = 0;
	private long timeMills = 0;
    private long eventDownTime = 0;
    private long eventUpTime = 0;
    private String lastListPath = "";
    private String trackName = "";
    private boolean showUrls;
    private boolean setDefaultCodePageOnDestroy = true;
    private ArrayList<String> names;
    private ArrayList<String> urls;
    private Toolbar actionBar;
    private ActionBarFragment actionBarFragment;
    private FragmentTransaction transaction;
    private Toast toast;
	private AlertDialog mDialog;
	private SharedPreferences sharedPreferences;
	private ListView list;
	private TextView trackNameTextViev;
	private Menu menu;
	private MenuItem menuOpenMain;
	private MenuItem menuSaveMain;
	private LinearLayout menuBar;
	private ImageButton btnEdit;
	private ImageButton btnOpen;
	private ImageButton btnOpenMain;
	private ImageButton btnDownload;
	private ImageButton btnSave;
	private ImageButton btnSaveMain;
	private ImageButton btnPrev;
	private ImageButton btnPlayStop;
	private ImageButton btnNext;
	private ImageButton btnToggleBar;
	private BroadcastReceiver receiver;
	private Animation trackShow;
	private Animation trackHide;
	private LayoutParams trackLayoutParams;
    private LayoutParams layoutParams;
    private Runnable runnable;
    private TelephonyManager telephonyManager;
    private SoundPool soundPool;
    private int sound1;
    private int sound2;
    private int sound3;
    private int sound4;

	//-----------------------------------------------------------------------------
	@Override
	protected void onCreate(Bundle savedInstanceState) {
        telephonyManager = (TelephonyManager)this.getSystemService(Context.TELEPHONY_SERVICE);
        sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        showUrls = sharedPreferences.getBoolean("list_show_url", false);
        headsetBeepEnable = sharedPreferences.getBoolean("headset_beep_enable", false);
        theme = ThemeChooser.getAppTheme(this);
        themeBaseColor = getResources().getColor(theme[0]);
        themeAccentColor = getResources().getColor(theme[1]);
        setTheme(theme[2]);
        runnable = new Runnable() {
            @Override
            public void run() {
                switch (clickCount) {
                    case 1:
                        if(headsetBeepEnable)beep(1);
                        playStop();
                        break;
                    case 2:
                        if(headsetBeepEnable)beep(2);
                        goNext();
                        break;
                    case 3:
                        if(headsetBeepEnable)beep(3);
                        goPrev();
                        break;
                }
                clickCount = 0;
            }
        };
        if(savedInstanceState != null) {
            dialogIsShow = savedInstanceState.getInt("dialogIsShow", DIALOG_NONE);
            editPosition = savedInstanceState.getInt("editPosition", editPosition);
            lastListPath = savedInstanceState.getString("lastListPath", lastListPath);;
        }
        if(headsetBeepEnable && soundPool == null) {
            soundPool = new SoundPool(1, AudioManager.STREAM_MUSIC, 0);
            sound1 = soundPool.load(this, R.raw.beep1, 1);
            sound2 = soundPool.load(this, R.raw.beep2, 1);
            sound3 = soundPool.load(this, R.raw.beep3, 1);
            sound4 = soundPool.load(this, R.raw.beep4, 1);
        }
        super.onCreate(savedInstanceState);
        setContentView(R.layout.radio_main);
        createActionBar(getResources().getConfiguration().orientation);
        list = (ListView) findViewById(R.id.list);
        brandGlowEffect(this, themeAccentColor);
        LinearLayout controller = (LinearLayout) findViewById(R.id.controller);
        controller.setBackgroundColor(themeBaseColor);
		list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@SuppressLint("DefaultLocale")
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				Intent i = new Intent(getApplicationContext(), RadioService.class)
				.putExtra("internal", true)
				.putExtra(getApplicationContext().getResources().getString(R.string.service_state), RadioService.POS)
				.putExtra(getApplicationContext().getResources().getString(R.string.media_position), position);
				getApplicationContext().startService(i);
			};
		});
		list.setOnItemLongClickListener(new AdapterView.OnItemLongClickListener() {
			@Override
			public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {
                editPosition = position;
				if(urls.get(position).startsWith("file:///data")) {
                    showToast(getResources().getString(R.string.ticker_playlist_cached), Toast.LENGTH_SHORT);
				} else {
					showChoseActionDialog(position);
				}
				return true;
			};
		});
		list.setOnScrollListener(new AbsListView.OnScrollListener () {

			@Override
			public void onScrollStateChanged(AbsListView view, int scrollState) {

			};

			@Override
			public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
				first = firstVisibleItem;
				last = first + visibleItemCount;
			};
	    	
	    });
		trackNameTextViev = (TextView) findViewById(R.id.track_name);
		menuBar = (LinearLayout) findViewById(R.id.menu_bar);
		btnToggleBar = (ImageButton) findViewById(R.id.btn_toggle_bar);
		btnToggleBar.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT) {
                    if (sharedPreferences.getBoolean("view_show_panel", false)) {
                        menuBar.setVisibility(View.GONE);
                        sharedPreferences.edit().putBoolean("view_show_panel", false).apply();
                    } else {
                        menuBar.setVisibility(View.VISIBLE);
                        sharedPreferences.edit().putBoolean("view_show_panel", true).apply();
                    }
                }
            }

            ;
        });
		btnToggleBar.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.ticker_btn_bar), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnEdit = (ImageButton) findViewById(R.id.btn_edit);
		btnEdit.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                editList();
            }

            ;
        });
		btnEdit.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_edit), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnOpen = (ImageButton) findViewById(R.id.btn_open);
		btnOpen.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                openList();
            }

            ;
        });
		btnOpen.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_open), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnOpenMain = (ImageButton) findViewById(R.id.btn_open_main);
		btnOpenMain.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                openMainList();
            }

            ;
        });
		btnOpenMain.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_open_main), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnDownload = (ImageButton) findViewById(R.id.btn_download);
		btnDownload.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                downloadList();
            }

            ;
        });
		btnDownload.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_download), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnSave = (ImageButton) findViewById(R.id.btn_save);
		btnSave.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                saveList();
            }

            ;
        });
		btnSave.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_save_as), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnSaveMain = (ImageButton) findViewById(R.id.btn_save_main);
		btnSaveMain.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                showSaveAsMainDialog();
            }

            ;
        });
		btnSaveMain.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.action_save_main), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnPlayStop = (ImageButton) findViewById(R.id.btn_play_stop);
		btnPlayStop.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                playStop();
            }

            ;
        });
		btnPlayStop.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.ticker_play), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnPrev = (ImageButton) findViewById(R.id.btn_prev);
		btnPrev.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                goPrev();
            }

            ;
        });
		btnPrev.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.ticker_prev), Toast.LENGTH_SHORT);
				return true;
			};
		});
		btnNext = (ImageButton) findViewById(R.id.btn_next);
		btnNext.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                goNext();
            }

            ;
        });
		btnNext.setOnLongClickListener(new OnLongClickListener() {
			@Override
			public boolean onLongClick(View v) {
				showToast(getResources().getString(R.string.ticker_next), Toast.LENGTH_SHORT);
				return true;
			};
		});
		if(sharedPreferences.getBoolean("view_show_panel", false)) {
			if(getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT) {
				menuBar.setVisibility(View.VISIBLE);
			} else {
				menuBar.setVisibility(View.GONE);
			}
		} else {
	    	menuBar.setVisibility(View.GONE);
		}
		trackLayoutParams = trackNameTextViev.getLayoutParams();
		trackHeight = trackLayoutParams.height;
		trackShow = AnimationUtils.loadAnimation(this, R.anim.track_show);
		trackShow.setAnimationListener(new Animation.AnimationListener(){
		    @Override
		    public void onAnimationStart(Animation arg0) {
				trackLayoutParams.height = trackHeight;
				trackNameTextViev.setLayoutParams(trackLayoutParams);
		    };           
		    @Override
		    public void onAnimationRepeat(Animation arg0) {
		    };          
		    @Override
		    public void onAnimationEnd(Animation arg0) {
				if(currentPosition > (last - 2)) list.setSelection(currentPosition);
		    };
		});
		trackHide = AnimationUtils.loadAnimation(this, R.anim.track_hide);
		trackHide.setAnimationListener(new Animation.AnimationListener(){
		    @Override
		    public void onAnimationStart(Animation arg0) {
		    };           
		    @Override
		    public void onAnimationRepeat(Animation arg0) {
		    };         
		    @Override
		    public void onAnimationEnd(Animation arg0) {
		    	trackLayoutParams.height = 0;
		    	trackNameTextViev.setLayoutParams(trackLayoutParams);
		    };
		});
		receiver = new BroadcastReceiver() {
			@Override
			public void onReceive(Context context, Intent intent) {
				lastListPath = intent.getStringExtra(getResources().getString(R.string.data_path));
				currentPosition = intent.getIntExtra(getResources().getString(R.string.media_position), 0);
				serviceState = intent.getIntExtra(getResources().getString(R.string.service_state), RadioService.TERM);
            	trackName = intent.getStringExtra(getResources().getString(R.string.media_track));
            	names = intent.getStringArrayListExtra(getResources().getString(R.string.media_name));
            	urls = intent.getStringArrayListExtra(getResources().getString(R.string.media_url));
            	boolean renewList = intent.getBooleanExtra(getResources().getString(R.string.reload_list), true);
            	switch (serviceState) {
					case RadioService.TERM:
					case RadioService.STOP:
						btnPlayStop.setImageResource(R.drawable.ic_media_play);
                        trackNameTextViev.startAnimation(trackHide);
						break;
					case RadioService.PLAY:
						btnPlayStop.setImageResource(R.drawable.ic_media_pause);
						if(trackName.length() > 0) {
							trackNameTextViev.setText(trackName);
                            trackNameTextViev.startAnimation(trackShow);
						} else {
                            trackNameTextViev.startAnimation(trackHide);
						}
						break;
					case RadioService.ERROR:
					case RadioService.LOAD:
						btnPlayStop.setImageResource(R.drawable.ic_media_pause);
                        trackNameTextViev.startAnimation(trackHide);
						break;
				};
				
				if(renewList) {
					initAdapter();
					getIntent().setData(null);
				} else {
					list.setItemChecked(currentPosition, true);
					if(currentPosition < first || currentPosition > (last - 2)) list.setSelection(currentPosition);
				};
				
				if(menu != null)renewOptionsMenu();
			};
		};
	    registerReceiver(receiver, new IntentFilter(getResources().getString(R.string.service_out_action)));
	};
	
	//-----------------------------------------------------------------------------
	private void initAdapter() {
		if(lastListPath.equalsIgnoreCase(Storage.getPath(this, Storage.MAIN_LIST))) {
            if(actionBar != null) {
                actionBar.setTitle(getText(R.string.app_name));
                actionBar.setSubtitle(getText(R.string.main_list));
            }
		} else {
			if(menuOpenMain != null) menuOpenMain.setVisible(true);
			if(menuSaveMain != null) menuSaveMain.setVisible(true);
            if(actionBar != null) {
                File file = new File(lastListPath);
                actionBar.setTitle(getText(R.string.app_name));
                actionBar.setSubtitle(file.getName());
            }
        }
		list.setAdapter(new AdapterRadioMain(this, names, urls, showUrls));
		list.setItemChecked(currentPosition, true);
		list.setSelection(currentPosition);
	};

	// -----------------------------------------------------------------------------
	private void terminateService() {
		sendBroadcast(new Intent(getResources().getString(R.string.media_close)));
	};

	// -----------------------------------------------------------------------------
	private void playStop() {
		this.startService(new Intent(this, RadioService.class)
		.putExtra("internal", true)
		.putExtra(this.getResources().getString(R.string.service_state), RadioService.PLAY));
	};
	
	// -----------------------------------------------------------------------------
	private void goPrev() {
		this.startService(new Intent(this, RadioService.class)
        .putExtra("internal", true)
		.putExtra(this.getResources().getString(R.string.service_state), RadioService.PREV));
	};

	// -----------------------------------------------------------------------------
	private void goNext() {
		this.startService(new Intent(this, RadioService.class)
        .putExtra("internal", true)
		.putExtra(this.getResources().getString(R.string.service_state), RadioService.NEXT));
	};
	
	// -----------------------------------------------------------------------------
	private void editList() {
        setDefaultCodePageOnDestroy = false;
		Intent i = new Intent(this, RadioEdit.class)
		.setData(Uri.parse("file:" + lastListPath))
		.putExtra("current_position", currentPosition);
        startActivityForResult(i, REQUEST_EDIT);
	};
	
	// -----------------------------------------------------------------------------
	private void openList() {
		if(sharedPreferences.getBoolean("use_external_open_dialog", false)) {
			showOpenDialogExternal();
		} else {
			showOpenDialog();
		}
	};
	
	// -----------------------------------------------------------------------------
	private void saveList() {
		String f = "";
		if(lastListPath.equalsIgnoreCase(Storage.getPath(this, Storage.MAIN_LIST))
		|| lastListPath.startsWith(Storage.getPath(this, Storage.TMP_DIR))
		|| lastListPath.equalsIgnoreCase("")) {
			f = Environment.getExternalStorageDirectory().getPath();
		} else {
			f = new File(lastListPath).getParent();
		};
	    startActivityForResult(new Intent(getBaseContext(), DialogFile.class)
		.putExtra(DialogFile.START_PATH, f)
		.putExtra(DialogFile.SAVED_NAME, new File(lastListPath).getName())
		.putExtra(DialogFile.CAN_SELECT_DIR, false)
		.putExtra(DialogFile.SELECTION_MODE, DialogFile.MODE_CREATE)
		.putExtra(DialogFile.FORMAT_FILTER, new String[] { "m3u", "pls" }), REQUEST_SAVE);
	};

	// -----------------------------------------------------------------------------
	private void openMainList() {
		startService(new Intent(this, RadioService.class)
		.putExtra(getResources().getString(R.string.service_state), RadioService.LOAD)
		.putExtra(getResources().getString(R.string.data_path), Storage.getPath(this, Storage.MAIN_LIST)));
	};
	
	// -----------------------------------------------------------------------------
	private void downloadList() {
        if(!isOnline(this)) {
            Intent intent = new Intent("android.intent.action.MAIN")
                    .setClass(this, DialogNetwork.class)
                    .setFlags(Intent.FLAG_ACTIVITY_NEW_TASK
                            | Intent.FLAG_ACTIVITY_NO_HISTORY
                            | Intent.FLAG_ACTIVITY_RETAIN_IN_RECENTS);
            startActivity(intent);
            return;
        }
        startActivity(new Intent(this, RadioDownloader.class)
		.setData(Uri.parse(getResources().getString(R.string.default_list_url)))
		.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_NO_HISTORY));
	};
	
	// -----------------------------------------------------------------------------
	private void createList() {
        String f = "";
        if(lastListPath.equalsIgnoreCase(Storage.getPath(this, Storage.MAIN_LIST))
                || lastListPath.startsWith(Storage.getPath(this, Storage.TMP_DIR))
                || lastListPath.equalsIgnoreCase("")) {
            f = Environment.getExternalStorageDirectory().getPath();
        } else {
            f = new File(lastListPath).getParent();
        };
	    startActivityForResult(new Intent(this, DialogFile.class)
		.putExtra(DialogFile.START_PATH, f)
		.putExtra(DialogFile.CAN_SELECT_DIR, true)
		.putExtra(DialogFile.SELECTION_MODE, DialogFile.MODE_SELECT_DIR)
	    .putExtra(DialogFile.FORMAT_FILTER, getResources().getStringArray(R.array.media_file_extensions_array)), REQUEST_CREATE);
	};

	// -----------------------------------------------------------------------------
	private void addToMain(final int pos) {
		Data mainList = new Data(this, Storage.getPath(this, Storage.MAIN_LIST));
		
		mainList.names.add(names.get(pos));
		mainList.urls.add(urls.get(pos));
	
		mainList.writeList();
	
		if (mainList.errorMessage.equalsIgnoreCase("")) {
			showToast("1 " + getText(R.string.dialog_add_message), Toast.LENGTH_LONG);
		} else {
			showToast(mainList.errorMessage, Toast.LENGTH_LONG);
		}
	};

	// -----------------------------------------------------------------------------
	private void createNumericShortcut(final int position) {
		Intent i = new Intent(this, RadioServiceActivity.class)
		.putExtra(getResources().getString(R.string.service_state), RadioService.SHORT)
		.putExtra(getResources().getString(R.string.media_position), position);
        DisplayMetrics metrics = getResources().getDisplayMetrics();
        float bigTextSize = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 24, metrics);
        float smallTextSize = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 18, metrics);
        float outlineWidth = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 3, metrics);
        Paint paint = new Paint();
        paint.setAntiAlias(true);
        paint.setColor(0xFFFFFFFF);
        Bitmap tmpBitmap = BitmapFactory.decodeResource(getResources(), R.drawable.shortcut_item);
        Bitmap bitmap = tmpBitmap.copy(Bitmap.Config.ARGB_8888, true);
        Canvas canvas = new Canvas(bitmap);
        int radius = canvas.getWidth() / 2;
        canvas.drawCircle(radius, radius, radius, paint);
        paint.setColor(themeBaseColor);
        canvas.drawCircle(radius, radius, radius - outlineWidth, paint);
        paint.setColor(0xFFFFFFFF);
        paint.setTypeface(Typeface.DEFAULT_BOLD);
	    paint.setTextAlign(Paint.Align.CENTER);
	    if(position > 98) {
            paint.setTextSize(smallTextSize);
	    } else {
            paint.setTextSize(bigTextSize);
	    }
        int yPos = (int) (radius - ((paint.descent() + paint.ascent()) / 2)) ;
	    canvas.drawText("" + (position + 1), radius, yPos, paint);
	    Intent a = new Intent()
	    .putExtra(Intent.EXTRA_SHORTCUT_INTENT, i)
	    .putExtra(Intent.EXTRA_SHORTCUT_NAME, getResources().getString(R.string.app_name))
	    .putExtra(Intent.EXTRA_SHORTCUT_ICON, bitmap)
	    .setAction("com.android.launcher.action.INSTALL_SHORTCUT");
	    sendBroadcast(a);
	};
	
	// -----------------------------------------------------------------------------
//	private void createPlayBtnShortcut() {
//		Intent i = new Intent(this, RadioServiceActivity.class)
//		.putExtra(getResources().getString(R.string.service_state), RadioService.PLAY);
//	 
//	    Intent a = new Intent()
//	    .putExtra(Intent.EXTRA_SHORTCUT_INTENT, i)
////	    .putExtra(Intent.EXTRA_SHORTCUT_NAME, getResources().getString(R.string.action_play))
//	    .putExtra(Intent.EXTRA_SHORTCUT_ICON_RESOURCE, Intent.ShortcutIconResource.fromContext(this, R.drawable.shortcut_btn_play))
//	    .setAction("com.android.launcher.action.INSTALL_SHORTCUT");
//	    sendBroadcast(a);
//	};
//	
//	// -----------------------------------------------------------------------------
//	private void createStopBtnShortcut() {
//		Intent i = new Intent(this, RadioServiceActivity.class)
//		.putExtra(getResources().getString(R.string.service_state), RadioService.STOP);
//	 
//	    Intent a = new Intent()
//	    .putExtra(Intent.EXTRA_SHORTCUT_INTENT, i)
////	    .putExtra(Intent.EXTRA_SHORTCUT_NAME, getResources().getString(R.string.action_stop))
//	    .putExtra(Intent.EXTRA_SHORTCUT_ICON_RESOURCE, Intent.ShortcutIconResource.fromContext(this, R.drawable.shortcut_btn_stop))
//	    .setAction("com.android.launcher.action.INSTALL_SHORTCUT");
//	    sendBroadcast(a);
//	};
//	
//	// -----------------------------------------------------------------------------
//	private void createPrevBtnShortcut() {
//		Intent i = new Intent(this, RadioServiceActivity.class)
//		.putExtra(getResources().getString(R.string.service_state), RadioService.PREV);
//	 
//	    Intent a = new Intent()
//	    .putExtra(Intent.EXTRA_SHORTCUT_INTENT, i)
////	    .putExtra(Intent.EXTRA_SHORTCUT_NAME, getResources().getString(R.string.action_prev))
//	    .putExtra(Intent.EXTRA_SHORTCUT_ICON_RESOURCE, Intent.ShortcutIconResource.fromContext(this, R.drawable.shortcut_btn_prev))
//	    .setAction("com.android.launcher.action.INSTALL_SHORTCUT");
//	    sendBroadcast(a);
//	};
//	
//	// -----------------------------------------------------------------------------
//	private void createNextBtnShortcut() {
//		Intent i = new Intent(this, RadioServiceActivity.class)
//		.putExtra(getResources().getString(R.string.service_state), RadioService.NEXT);
//	 
//	    Intent a = new Intent()
//	    .putExtra(Intent.EXTRA_SHORTCUT_INTENT, i)
////	    .putExtra(Intent.EXTRA_SHORTCUT_NAME, getResources().getString(R.string.action_next))
//	    .putExtra(Intent.EXTRA_SHORTCUT_ICON_RESOURCE, Intent.ShortcutIconResource.fromContext(this, R.drawable.shortcut_btn_next))
//	    .setAction("com.android.launcher.action.INSTALL_SHORTCUT");
//	    sendBroadcast(a);
//	};
	
	// -----------------------------------------------------------------------------
	private void showOpenDialog() {
		String f = "";
		if(lastListPath.equalsIgnoreCase(Storage.getPath(this, Storage.MAIN_LIST))
		|| lastListPath.startsWith(Storage.getPath(this, Storage.TMP_DIR))
		|| lastListPath.equalsIgnoreCase("")) {
			f = Environment.getExternalStorageDirectory().getPath();
		} else {
			f = new File(lastListPath).getParent();
		};
	    startActivityForResult(new Intent(this, DialogFile.class)
		.putExtra(DialogFile.START_PATH, f)
		.putExtra(DialogFile.CAN_SELECT_DIR, false)
		.putExtra(DialogFile.SELECTION_MODE, DialogFile.MODE_OPEN)
	    .putExtra(DialogFile.FORMAT_FILTER, new String[] { "m3u", "pls", "ram" }), REQUEST_OPEN);
	}

	// -----------------------------------------------------------------------------
	private void showOpenDialogExternal() {
	    Intent intent = new Intent(Intent.ACTION_GET_CONTENT)
	    .putExtra(DialogFile.FORMAT_FILTER, new String[] { "m3u", "pls", "ram" })
	    .setType("application/x-mpegURL, application/vnd.apple.mpegurl, audio/x-scpls") 
	    .addCategory(Intent.CATEGORY_OPENABLE);
	    if (intent.resolveActivity(getPackageManager()) != null) {
	        startActivityForResult(intent, REQUEST_OPEN);
	    } else {
	    	showOpenDialog();
		}
	}

	// -----------------------------------------------------------------------------
	private void showChoseActionDialog(final int pos) {
		String[] items;
	    final boolean isMain;
		if (lastListPath.equalsIgnoreCase(Storage.getPath(this, Storage.MAIN_LIST))) {
			isMain = true;
			items = getApplicationContext().getResources().getStringArray(R.array.dialog_chose_action_array_main);
		} else {
			isMain = false;
			items = getApplicationContext().getResources().getStringArray(R.array.dialog_chose_action_array);
		}
		LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_choose, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
		TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.dialog_chose_action_title));
        layoutParams = dialogTitle.getLayoutParams();
        layoutParams.width = (int)(getScreenMinPixels()/100*90);
        dialogTitle.setLayoutParams(layoutParams);
		ListView list = (ListView) dialog.findViewById(android.R.id.list);
		list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		list.setAdapter(new AdapterActionList(this, items, isMain));
		list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
	        	switch (position) {
	        	case 0:
					Intent i = new Intent(getApplicationContext(), RadioService.class)
					.putExtra(getApplicationContext().getResources().getString(R.string.service_state), RadioService.EXT)
					.putExtra(getApplicationContext().getResources().getString(R.string.media_position), pos);
					getApplicationContext().startService(i);
	    			return;
	        	case 1:
					if(isMain) {
						createNumericShortcut(pos);
					} else {
						addToMain(pos);
					}
	        		return;
	        	}
			}
		});
        Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
            };
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
            };
        });
        mDialog.show();
        dialogIsShow = DIALOG_CHOOSE_ACTION;
	};

    // -----------------------------------------------------------------------------
    private void showChangeCodePageDialog() {
        final CodePage codePage = new CodePage(this);
        int selectedPosition = codePage.getIndex();
        LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_choose, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
        TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.action_code_page));
        ListView list = (ListView) dialog.findViewById(android.R.id.list);
        if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
            layoutParams = list.getLayoutParams();
            layoutParams.height = (int)(getScreenMinPixels()/100*60);
            list.setLayoutParams(layoutParams);
        } else {
            layoutParams = list.getLayoutParams();
            layoutParams.height = (int)(getScreenMaxPixels()/100*70);
            list.setLayoutParams(layoutParams);
        }
        list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
        list.setAdapter(new AdapterDialogChooseValue(this, codePage.getNames(), selectedPosition));
        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                codePage.setIndex(position);
                startService(new Intent(getApplicationContext(), RadioService.class)
                        .putExtra(getResources().getString(R.string.service_state), RadioService.LOAD)
                        .putExtra(getResources().getString(R.string.data_path), lastListPath));
            }
        });
        Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
            }
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
            };
        });
        mDialog.show();
        list.setSelection(selectedPosition);
        dialogIsShow = DIALOG_CODE_PAGE;
    };

	// -----------------------------------------------------------------------------
	private void showSaveAsMainDialog() {
        LinearLayout dialog = (LinearLayout) getLayoutInflater().inflate(R.layout.dialog_alert, new LinearLayout(this), false);
        dialog.setBackgroundResource(theme[3]);
		TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
        dialogTitle.setText(getText(R.string.dialog_save_as_main_title));
//        layoutParams = dialogTitle.getLayoutParams();
//        layoutParams.width = (int)(getScreenMinPixels()/100*90);
//        dialogTitle.setLayoutParams(layoutParams);
		TextView dialogMessage = (TextView) dialog.findViewById(R.id.dialog_message);
		dialogMessage.setText(getText(R.string.dialog_save_as_main_message));
        Button btnOk = (Button) dialog.findViewById(R.id.btn_ok);
        btnOk.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
                getApplicationContext().startService(new Intent(getApplicationContext(), RadioService.class)
                        .putExtra(getApplicationContext().getResources().getString(R.string.service_state), RadioService.SAVE)
                        .putExtra(getApplicationContext().getResources().getString(R.string.data_path), Storage.getPath(getApplicationContext(), Storage.MAIN_LIST)));
            };
        });
        Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                dialogIsShow = DIALOG_NONE;
                if(mDialog != null) mDialog.dismiss();
                mDialog = null;
            };
        });
        AlertDialog.Builder builder = new AlertDialog.Builder(this, theme[4]);
        builder.setView(dialog).setCancelable(true);
        mDialog = builder.create();
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialogIsShow = DIALOG_NONE;
                mDialog = null;
            };
        });
        mDialog.show();
        dialogIsShow = DIALOG_SAVE_AS_MAIN;
	};

	// -----------------------------------------------------------------------------
	private void portraitOptionsMenu() {
		if(sharedPreferences.getBoolean("view_show_panel", false)) {
	    	menuBar.setVisibility(View.VISIBLE);
		} else {
	    	menuBar.setVisibility(View.GONE);
		}
		btnToggleBar.setVisibility(View.VISIBLE);
	    menu.clear();
	    getMenuInflater().inflate(R.menu.radio_main_portrait, menu);
		menuOpenMain = menu.findItem(R.id.action_open_main);
		menuSaveMain = menu.findItem(R.id.action_save_main);
		menu.findItem(R.id.action_settings).setIntent(new Intent(this, RadioSettings.class));
		renewOptionsMenu();
	};

	// -----------------------------------------------------------------------------
	private void landscapeOptionsMenu() {
		btnToggleBar.setVisibility(View.GONE);
    	menuBar.setVisibility(View.GONE);
	    menu.clear();    
	    getMenuInflater().inflate(R.menu.radio_main_landscape, menu);
		menuOpenMain = menu.findItem(R.id.action_open_main);
		menuSaveMain = menu.findItem(R.id.action_save_main);
		menu.findItem(R.id.action_settings).setIntent(new Intent(this, RadioSettings.class));
		renewOptionsMenu();
	};
	
	// -----------------------------------------------------------------------------
	private void renewOptionsMenu() {
		if (lastListPath.equalsIgnoreCase(Storage.getPath(this, Storage.MAIN_LIST))) {
			menuOpenMain.setVisible(false);
			menuSaveMain.setVisible(false);
			btnOpenMain.setVisibility(View.GONE);
			btnSaveMain.setVisibility(View.GONE);
		} else {
			menuOpenMain.setVisible(true);
			menuSaveMain.setVisible(true);
			btnOpenMain.setVisibility(View.VISIBLE);
			btnSaveMain.setVisibility(View.VISIBLE);
		}
	};
	
	// -----------------------------------------------------------------------------
	private void showToast(String message, int length) {
		toast = Toast.makeText(this, message, length);
		toast.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL, 0, 0);
		toast.show();
	};

    // -----------------------------------------------------------------------------
    private void createActionBar(int orientation) {
        actionBarFragment = new ActionBarFragment();
        actionBarFragment.setParentActivity(this);
        Bundle args = new Bundle();
        if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
            args.putInt(actionBarFragment.LAYOUT_ORIENTATION, actionBarFragment.LAYOUT_LANDSCAPE);
        } else {
            args.putInt(actionBarFragment.LAYOUT_ORIENTATION, actionBarFragment.LAYOUT_PORTRAIT);
        }
        args.putString(actionBarFragment.TITLE, getResources().getString(R.string.app_name));
        args.putString(actionBarFragment.SUBTITLE, getResources().getString(R.string.app_description));
        actionBarFragment.setArguments(args);
        transaction = getSupportFragmentManager().beginTransaction();
        transaction.replace(R.id.action_bar_container, actionBarFragment);
        transaction.commit();
    };

	//-----------------------------------------------------------------------------
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		this.menu = menu;
        actionBar = actionBarFragment.getActionBar();
		orientation = getResources().getConfiguration().orientation;
		if(orientation == Configuration.ORIENTATION_PORTRAIT) {
	    	portraitOptionsMenu();
		} else if(orientation == Configuration.ORIENTATION_LANDSCAPE) {
	    	landscapeOptionsMenu();
	    }
        switch(dialogIsShow) {
            case DIALOG_SAVE_AS_MAIN:
                showSaveAsMainDialog();
                break;
            case DIALOG_CHOOSE_ACTION:
                showChoseActionDialog(editPosition);
                break;
            case DIALOG_CODE_PAGE:
                showChangeCodePageDialog();
                break;
        }
        startService(new Intent(this, RadioService.class).putExtra(getResources().getString(R.string.service_state), RadioService.STATE));
		return true;
	}

	//-----------------------------------------------------------------------------
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch(item.getItemId()) {
			case R.id.action_edit:
				editList();
				return true;
			case R.id.action_open:
				openList();
				return true;
			case R.id.action_open_main:
				openMainList();
				return true;
			case R.id.action_save_as:
				saveList();
				return true;
			case R.id.action_save_main:
				showSaveAsMainDialog();
				return true;
			case R.id.action_download_default:
				downloadList();
				return true;
			case R.id.action_create:
				createList();
				return true;
            case R.id.action_code_page:
                showChangeCodePageDialog();
                return true;
			case R.id.action_quit:
				terminateService();
				finish();
				return true;
		};
		return super.onOptionsItemSelected(item);
	};

    // -----------------------------------------------------------------------------
    private void beep(int id) {
        switch(id) {
            case 1:
                soundPool.play(sound1, 1.0f, 1.0f, 1000, 0, 1.0f);
                break;
            case 2:
                soundPool.play(sound2, 1.0f, 1.0f, 1000, 0, 1.0f);
                break;
            case 3:
                soundPool.play(sound3, 1.0f, 1.0f, 1000, 0, 1.0f);
                break;
            case 4:
                soundPool.play(sound4, 1.0f, 1.0f, 1000, 0, 1.0f);
                break;
        }
    };

    // -----------------------------------------------------------------------------
    @Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
        if(telephonyManager.getCallState() == TelephonyManager.CALL_STATE_IDLE) {
            if(keyCode == KeyEvent.KEYCODE_BACK) {
                dialogIsShow = DIALOG_NONE;
                String path = ListStack.pull(getApplicationContext());
                if(path.equalsIgnoreCase("")) {
                    if(!getIntent().hasExtra("internal")) {
                        long time = System.currentTimeMillis();
                        if((time - timeMills) < 2000) {
                            if(toast != null) toast.cancel();
                            finish();
                        } else {
                            timeMills = time;
                            showToast(getResources().getString(R.string.ticker_press_to_exit), Toast.LENGTH_SHORT);
                        }
                    } else {
                        finish();
                    }
                    return true;
                } else {
                    startService(new Intent(this, RadioService.class)
                            .putExtra(getResources().getString(R.string.service_state), RadioService.LOAD)
                            .putExtra(getResources().getString(R.string.data_path), path));
                    return true;
                }
            } else {
                switch (keyCode) {
                    case KeyEvent.KEYCODE_HEADSETHOOK:
                        clickCount++;
                        eventDownTime = event.getDownTime();
                        if (clickCount == 1) {
                            return super.onKeyDown(keyCode, event);
                        }
                        return true;
                }
            }
        }
		return super.onKeyDown(keyCode, event);
	};

    // -----------------------------------------------------------------------------
    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if(telephonyManager.getCallState() == TelephonyManager.CALL_STATE_IDLE) {
            switch (keyCode) {
                case KeyEvent.KEYCODE_HEADSETHOOK:
                    eventUpTime = event.getEventTime();
                    if (eventDownTime == eventUpTime) {
                        if(headsetBeepEnable)beep(1);
                        if (clickCount == 1) {
                            new Handler().postDelayed(runnable, 3000);
                        }
                    } else {
                        if (clickCount == 1) {
                            new Handler().postDelayed(runnable, 500);
                        }
                    }
                    return true;
                case KeyEvent.KEYCODE_MEDIA_PLAY_PAUSE:
                case KeyEvent.KEYCODE_MEDIA_PLAY:
                case KeyEvent.KEYCODE_MUTE:
                case KeyEvent.KEYCODE_MEDIA_PAUSE:
                case KeyEvent.KEYCODE_MEDIA_STOP:
                    playStop();
                    return true;
                case KeyEvent.KEYCODE_MEDIA_FAST_FORWARD:
                case KeyEvent.KEYCODE_MEDIA_NEXT:
                    if(headsetBeepEnable)beep(2);
                    goNext();
                    return true;
                case KeyEvent.KEYCODE_MEDIA_REWIND:
                case KeyEvent.KEYCODE_MEDIA_PREVIOUS:
                    if(headsetBeepEnable)beep(3);
                    goPrev();
                    return true;
            }
        }
        return super.onKeyUp(keyCode, event);
    };

	// -----------------------------------------------------------------------------
	@SuppressLint("DefaultLocale")
	protected void onActivityResult(final int requestCode, int resultCode, final Intent intent) {
		if (resultCode == Activity.RESULT_OK) {
			switch(requestCode) {
			case REQUEST_EDIT:
				startService(new Intent(this, RadioService.class)
				.putExtra(getResources().getString(R.string.service_state), RadioService.STATE));
                setDefaultCodePageOnDestroy = true;
                break;
			case REQUEST_OPEN:
                String checkPath = intent.getData().getPath().toLowerCase();
                if(checkPath.endsWith(".m3u") || checkPath.endsWith(".pls")) {
                    Storage.clearCache(this);
                    startService(new Intent(this, RadioService.class)
                    .putExtra(getResources().getString(R.string.service_state), RadioService.LOAD)
                    .putExtra(getResources().getString(R.string.data_path), intent.getData().getPath()));
                } else {
                    showToast(getResources().getString(R.string.ticker_error) + "\n" + intent.getData().getPath(), Toast.LENGTH_LONG);
                }
				break;
			case REQUEST_SAVE:
                startService(new Intent(this, RadioService.class)
				.putExtra(getResources().getString(R.string.service_state), RadioService.SAVE)
				.putExtra(getResources().getString(R.string.data_path), intent.getData().getPath()));
				break;
			case REQUEST_CREATE:
                Storage.clearCache(this);
                startActivityForResult(new Intent(this, RadioListCreator.class)
                        .setData(Uri.parse("file://" + intent.getData().getPath())), REQUEST_GET_LIST);
				break;
            case REQUEST_GET_LIST:
                startService(new Intent(this, RadioService.class)
                        .putExtra(getResources().getString(R.string.service_state), RadioService.LOAD)
                        .putExtra(getResources().getString(R.string.data_path), intent.getData().getPath()));
                break;
			};
		};
	};
	
    // -----------------------------------------------------------------------------
    private void brandGlowEffect(Context context, int brandColor) {
        if (Build.VERSION.SDK_INT >= 21)
        {
            EdgeEffect edgeEffectTop = new EdgeEffect(this);
            edgeEffectTop.setColor(brandColor);
            EdgeEffect edgeEffectBottom = new EdgeEffect(this);
            edgeEffectBottom.setColor(brandColor);
            try {
                Field f1 = AbsListView.class.getDeclaredField("mEdgeGlowTop");
                f1.setAccessible(true);
                f1.set(list, edgeEffectTop);

                Field f2 = AbsListView.class.getDeclaredField("mEdgeGlowBottom");
                f2.setAccessible(true);
                f2.set(list, edgeEffectBottom);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        else {
            int glowDrawableId = context.getResources().getIdentifier("overscroll_glow", "drawable", "android");
            Drawable androidGlow = context.getResources().getDrawable(glowDrawableId);
            androidGlow.setColorFilter(brandColor, PorterDuff.Mode.SRC_IN);
            int edgeDrawableId = context.getResources().getIdentifier("overscroll_edge", "drawable", "android");
            Drawable androidEdge = context.getResources().getDrawable(edgeDrawableId);
            androidEdge.setColorFilter(brandColor, PorterDuff.Mode.SRC_IN);
        }
    };

    // -----------------------------------------------------------------------------
    private static boolean isOnline(Context context)
    {
        ConnectivityManager cm = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo netInfo = cm.getActiveNetworkInfo();
        if (netInfo != null && netInfo.isConnectedOrConnecting())
        {
            return true;
        }
        return false;
    };

    // -----------------------------------------------------------------------------
    private int getScreenMaxPixels() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height >= width) {
            return height;
        } else {
            return width;
        }
    };

    // -----------------------------------------------------------------------------
    private int getScreenMinPixels() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height <= width) {
            return height;
        } else {
            return width;
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        if(hasFocus) {
            running = true;
        } else {
            running = false;
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt("dialogIsShow", dialogIsShow);
        outState.putInt("editPosition", editPosition);
        outState.putString("lastListPath", lastListPath);
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onPause() {
        running = false;
        super.onPause();
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onResume() {
        running = true;
        headsetBeepEnable = sharedPreferences.getBoolean("headset_beep_enable", false);
        super.onResume();
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onStop() {
        running = false;
        super.onStop();
    };

    // -----------------------------------------------------------------------------
    @Override
    protected void onStart() {
        running = true;
        headsetBeepEnable = sharedPreferences.getBoolean("headset_beep_enable", false);
        super.onStart();
    };

    // -----------------------------------------------------------------------------
	@Override
	protected void onDestroy() {
        running = false;
        if(mDialog != null) {
            mDialog.dismiss();
        }
        dialogIsShow = DIALOG_NONE;
        unregisterReceiver(receiver);
        if(setDefaultCodePageOnDestroy) {
            new CodePage(this).setDefault();
        }
		super.onDestroy();
	};
};
